/************************************************************************
 *
 *  ConverterFactory.java
 *
 *  Copyright: 2002-2018 by Henrik Just
 *
 *  This file is part of Writer2LaTeX.
 *  
 *  Writer2LaTeX is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Writer2LaTeX is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Writer2LaTeX.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Version 2.0 (2018-03-06)
 *
 */
 
package w2phtml.api;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/** This is a factory class which provides static methods to create converters
 *  for documents in OpenDocument (or OpenOffice.org 1.x) format into a specific MIME type
 */
public class ConverterFactory {

    // Version information
   private static final String propPath = "w2phtml/project.properties";
    /** Return the Writer2LaTeX version in the form
     *  (major version).(minor version).(patch level)<br/>
     *  Development versions have an odd minor version number
     *  @return the version number
     */
  	public static String getVersion() {
  		Properties prop = new Properties();
  		ClassLoader loader = Thread.currentThread().getContextClassLoader();
  		InputStream stream = loader.getResourceAsStream(propPath);
  		String version = "Failed to get version";
  		try {
  			prop.load(stream);
  			version = (String) prop.get("releaseVersion");
  		} catch (IOException e) {
  			// TODO Auto-generated catch block
  			e.printStackTrace();
  		}
  		return version;
  	}

    /** Return date information
     *  @return the release date for this Writer2LaTeX version
     */
  	public static String getDate() {
  		Properties prop = new Properties();
  		ClassLoader loader = Thread.currentThread().getContextClassLoader();
  		InputStream stream = loader.getResourceAsStream(propPath);
  		String date = "Failed to get date";
  		try {
  			prop.load(stream);
  			date = (String) prop.get("releaseDate");
  		} catch (IOException e) {
  			// TODO Auto-generated catch block
  			e.printStackTrace();
  		}
  		return date;
  	}

    /** <p>Create a <code>Converter</code> implementation which supports
     *  conversion into the specified MIME type.</p>
     *  <p>Currently supported MIME types are:</p>
     *  <ul>
     *    <li><code>text/html</code> for XHTML 1.0 strict format</li>
     *    <li><code>application/xhtml11</code> for XHTML 1.1 format
     *    Note that this is <em>not</em> the recommended media type for XHTML 1.1
     *    (see http://www.w3.org/TR/xhtml-media-types/), but it is used internally
     *    by Writer2xhtml to distinguish from XHTML+MathML</li>
     *    <li><code>application/xhtml+xml</code> for XHTML+MathML</li>
     *    <li><code>text/html5</code> for HTML5 documents
     *    Note that this is <em>not</em> the recommended media type for HTML5
     *    (see http://wiki.whatwg.org/), but it is used internally
     *    by Writer2xhtml to distinguish from HTML5</li>
     *    <li><code>application/epub+zip</code></li> for EPUB format</li>
     *    <li><code>epub3</code> for EPUB 3 format</li>
     *  </ul>
     *  
     *  @param sMIME the MIME type of the target format
     *  @return the required <code>Converter</code> or null if a converter for
     *  the requested MIME type could not be created
     */
    public static Converter createConverter(String sMIME) {
        Object converter = null;
        if (MIMETypes.XHTML.equals(sMIME)) {
            converter = createInstance("w2phtml.xhtml.Xhtml10Converter");
        }
        else if (MIMETypes.XHTML11.equals(sMIME)) {
            converter = createInstance("w2phtml.xhtml.Xhtml11Converter");
        }
        else if (MIMETypes.XHTML_MATHML.equals(sMIME)) {
            converter = createInstance("w2phtml.xhtml.XhtmlMathMLConverter");
        }
        else if (MIMETypes.HTML5.equals(sMIME)) {
            converter = createInstance("w2phtml.xhtml.Html5Converter");
        }
        else if (MIMETypes.EPUB3.equals(sMIME)) {
            converter = createInstance("w2phtml.epub.EPUB3Converter");
        }
        else if (MIMETypes.EPUB.equals(sMIME)) {
            converter = createInstance("w2phtml.epub.EPUBConverter");
        }
        else if (MIMETypes.RDF.equals(sMIME)) {
          converter = createInstance("w2phtml.rdf.RDFConverter");
        }
        return converter instanceof Converter ? (Converter) converter : null;
    }
	
    /** <p>Create a <code>BatchConverter</code> implementation which supports
     *  conversion into the specified MIME type</p>
     *  <p>The only currently supported MIME type is <code>text/html</code>
     *  (XHTML 1.0 strict)</p>
     *
     *  @param sMIME the MIME type of the target format
     *  @return the required <code>BatchConverter</code> or null if a converter
     *  for the requested MIME type could not be created
     */
    public static BatchConverter createBatchConverter(String sMIME) {
        Object converter = null;
        if (MIMETypes.XHTML.equals(sMIME)) {
            converter = createInstance("w2phtml.xhtml.BatchConverterImpl");
        }
        return converter instanceof BatchConverter ? (BatchConverter) converter : null;
    }
	
    /** Create a <code>StarMathConverter</code> implementation
     *
     *  @return the converter
     */
    public static StarMathConverter createStarMathConverter() {
        Object converter = createInstance("w2phtml.latex.StarMathConverter");
        return converter instanceof StarMathConverter ? (StarMathConverter) converter : null;
    }
	
    private static Object createInstance(String sClassName) {
        try {
		    return Class.forName(sClassName).newInstance();
        }
        catch (java.lang.ClassNotFoundException e) {
            return null;
        } 
        catch (java.lang.InstantiationException e) {
            return null;
        }
        catch (java.lang.IllegalAccessException e) {
            return null;
        } 
    }

}
